diff --git a/Makefile b/Makefile
index 5693fdb..672945b 100644
--- a/Makefile
+++ b/Makefile
@@ -55,6 +55,14 @@ SRCS += $(ALTIVECSRC)
 $(ALTIVECSRC:%.c=%.o): CFLAGS += $(ALTIVECFLAGS)
 endif
 
+# Cell Broadband Engine optims
+ifeq ($(HAVE_LIBSPE2),1)
+SRCS += common/ppc/cell_be/mc.c common/ppc/cell_be/spe_help.c
+SPUSRCS = common/ppc/cell_be/hpel_filter.spuc
+SPUOBJS = $(SPUSRCS:%.spuc=%)
+OBJEXTRA += $(SPUOBJS:%=%_csf.o)
+endif
+
 # VIS optims
 ifeq ($(ARCH),UltraSparc)
 ASMSRC += common/sparc/pixel.asm
@@ -77,10 +85,16 @@ libx264.a: .depend $(OBJS) $(OBJASM)
 	ar rc libx264.a $(OBJS) $(OBJASM)
 	ranlib libx264.a
 
+$(SPUOBJS): $(SPUSRCS)
+	$(SPU_CC) $(SPU_CFLAGS) -o $@ -x c $+
+
+$(OBJEXTRA): $(SPUOBJS)
+	$(EMBEDSPU) hpel_filter_handle $+ $@
+
 $(SONAME): .depend $(OBJS) $(OBJASM)
 	$(CC) -shared -o $@ $(OBJS) $(OBJASM) $(SOFLAGS) $(LDFLAGS)
 
-x264$(EXE): $(OBJCLI) libx264.a 
+x264$(EXE): $(OBJCLI) libx264.a $(OBJEXTRA)
 	$(CC) -o $@ $+ $(LDFLAGS)
 
 checkasm: tools/checkasm.o libx264.a
@@ -135,7 +149,7 @@ fprofiled:
 endif
 
 clean:
-	rm -f $(OBJS) $(OBJASM) $(OBJCLI) $(SONAME) *.a x264 x264.exe .depend TAGS
+	rm -f $(OBJS) $(OBJASM) $(OBJCLI) $(SPUOBJS) $(OBJEXTRA) $(SONAME) *.a x264 x264.exe .depend TAGS
 	rm -f checkasm checkasm.exe tools/checkasm.o
 	rm -f $(SRC2:%.c=%.gcda) $(SRC2:%.c=%.gcno)
 	- sed -e 's/ *-fprofile-\(generate\|use\)//g' config.mak > config.mak2 && mv config.mak2 config.mak
diff --git a/common/cpu.c b/common/cpu.c
index 2d722c6..3a74150 100644
--- a/common/cpu.c
+++ b/common/cpu.c
@@ -37,7 +37,7 @@
 #include "cpu.h"
 
 const x264_cpu_name_t x264_cpu_names[] = {
-    {"Altivec", X264_CPU_ALTIVEC},
+    {"AltiVec/VMX", X264_CPU_ALTIVEC},
 //  {"MMX",     X264_CPU_MMX}, // we don't support asm on mmx1 cpus anymore
     {"MMX2",    X264_CPU_MMX|X264_CPU_MMXEXT},
     {"MMXEXT",  X264_CPU_MMX|X264_CPU_MMXEXT},
@@ -50,6 +50,7 @@ const x264_cpu_name_t x264_cpu_names[] = {
     {"PHADD",   X264_CPU_MMX|X264_CPU_MMXEXT|X264_CPU_SSE|X264_CPU_SSE2|X264_CPU_SSE3|X264_CPU_SSSE3|X264_CPU_PHADD_IS_FAST},
     {"SSE4.1",  X264_CPU_MMX|X264_CPU_MMXEXT|X264_CPU_SSE|X264_CPU_SSE2|X264_CPU_SSE3|X264_CPU_SSSE3|X264_CPU_SSE4},
     {"SSE4.2",  X264_CPU_MMX|X264_CPU_MMXEXT|X264_CPU_SSE|X264_CPU_SSE2|X264_CPU_SSE3|X264_CPU_SSSE3|X264_CPU_SSE4|X264_CPU_SSE42},
+    {"CellSPU", X264_CPU_CELL_BE},
     {"Cache32", X264_CPU_CACHELINE_32},
     {"Cache64", X264_CPU_CACHELINE_64},
     {"Slow_mod4_stack", X264_CPU_STACK_MOD4},
@@ -205,8 +206,10 @@ uint32_t x264_cpu_detect( void )
 }
 
 #elif defined( SYS_LINUX )
+#ifndef HAVE_LIBSPE2
 #include <signal.h>
 #include <setjmp.h>
+
 static sigjmp_buf jmpbuf;
 static volatile sig_atomic_t canjump = 0;
 
@@ -244,6 +247,12 @@ uint32_t x264_cpu_detect( void )
 
     return X264_CPU_ALTIVEC;
 }
+#else
+uint32_t x264_cpu_detect( void )
+{
+    return X264_CPU_CELL_BE | X264_CPU_ALTIVEC;
+}
+#endif
 #endif
 
 #else
@@ -261,9 +270,16 @@ void x264_emms( void )
 }
 #endif
 
+#if defined(HAVE_LIBSPE2)
+#include <libspe2.h>
 
 int x264_cpu_num_processors( void )
 {
+    return spe_cpu_info_get(SPE_COUNT_USABLE_SPES, -1);
+}
+#else
+int x264_cpu_num_processors( void )
+{
 #if !defined(HAVE_PTHREAD)
     return 1;
 
@@ -298,3 +314,4 @@ int x264_cpu_num_processors( void )
     return 1;
 #endif
 }
+#endif
diff --git a/common/ppc/cell_be/hpel_filter.spuc b/common/ppc/cell_be/hpel_filter.spuc
new file mode 100644
index 0000000..f1d7e23
--- /dev/null
+++ b/common/ppc/cell_be/hpel_filter.spuc
@@ -0,0 +1,7 @@
+#include <stdio.h>
+
+int main(unsigned long long spe_id, unsigned long long argvp)
+{
+	printf("\nin hpel_filter");
+	return 0;
+}
diff --git a/common/ppc/cell_be/mc.c b/common/ppc/cell_be/mc.c
new file mode 100644
index 0000000..41dde9e
--- /dev/null
+++ b/common/ppc/cell_be/mc.c
@@ -0,0 +1,26 @@
+#include <stdint.h>
+#include <stdarg.h>
+#include <libspe2.h>
+
+#include "spe_help.h"
+#include "common/mc.h"
+#include "mc.h"
+
+typedef struct {
+	int i_height SPE_ALIGN;
+	int i_width SPE_ALIGN;
+	int i_string SPE_ALIGN;
+} x264_hpel_filter_cell_params_t;
+typedef x264_hpel_filter_cell_params_t *x264_hpel_filter_cell_params_ptr_t;
+
+extern spe_program_handle_t hpel_filter_handle;
+
+void x264_hpel_filter_cell_spu( uint8_t *dsth, uint8_t *dstv, uint8_t *dstc, uint8_t *src,
+                               int i_stride, int i_width, int i_height )
+{
+}
+
+void x264_mc_cell_spu_init( x264_mc_functions_t *pf )
+{
+	pf->hpel_filter = x264_hpel_filter_cell_spu;
+}
diff --git a/common/ppc/cell_be/mc.h b/common/ppc/cell_be/mc.h
new file mode 100644
index 0000000..bbaa1ff
--- /dev/null
+++ b/common/ppc/cell_be/mc.h
@@ -0,0 +1,6 @@
+#ifndef _PPC_CELL_BE_MC_H
+#define _PPC_CELL_BE_MC_H 1
+
+void x264_mc_cell_spu_init( x264_mc_functions_t *pf );
+
+#endif
diff --git a/common/ppc/cell_be/spe_help.c b/common/ppc/cell_be/spe_help.c
new file mode 100644
index 0000000..5b978f0
--- /dev/null
+++ b/common/ppc/cell_be/spe_help.c
@@ -0,0 +1,9 @@
+#include <stdlib.h>
+#include "spe_help.h"
+
+void *spe_aligned_malloc(unsigned int size)
+{
+	void *data;
+	posix_memalign(&data, SPE_ALIGNMENT, ROUND_UP_ALIGN(size, SPE_ALIGNMENT));
+	return data;
+}
diff --git a/common/ppc/cell_be/spe_help.h b/common/ppc/cell_be/spe_help.h
new file mode 100644
index 0000000..a1001a3
--- /dev/null
+++ b/common/ppc/cell_be/spe_help.h
@@ -0,0 +1,27 @@
+/* spe_help.h */
+
+#ifndef _SPE_HELP_H
+#define _SPE_HELP_H 1
+
+#include <stdlib.h>
+
+/* Alignment macros */
+#define SPE_ALIGNMENT 16
+#define SPE_ALIGNMENT_FULL 128
+#define SPE_ALIGN __attribute__((aligned(16)))
+#define SPE_ALIGN_FULL __attribute((aligned(128)))
+#define ROUND_UP_ALIGN(value, alignment)\
+  (((value) + ((alignment) - 1))&(~((alignment)-1)))
+
+#define malloc(x) spe_aligned_malloc((x))
+
+#ifndef __SPU__
+
+#define SPE_DEREF_UINT32(base, offset) *((unsigned int *)(((char *)(base)) + (offset)))
+
+void *spe_aligned_malloc(unsigned int size);
+
+#endif
+
+#endif
+
diff --git a/configure b/configure
index 6c5f949..bae3e53 100755
--- a/configure
+++ b/configure
@@ -259,6 +259,21 @@ case $host_cpu in
     fi
     ;;
   powerpc|powerpc64)
+    if [ $SYS = LINUX -a "$host_cpu" = powerpc64 ]; then
+      CFLAGS="$CFLAGS -m64"
+      LDFLAGS="$LDFLAGS -m64"
+      HAVE_LIBSPE2=0
+      if cc_check libspe2.h -lspe2 "spe_event_handler_create();" ; then
+	CC=`which ppu-gcc`
+        CFLAGS="$CFLAGS -DHAVE_LIBSPE2"
+        LDFLAGS="$LDFLAGS -lspe2"
+        HAVE_LIBSPE2=1
+        SPU_CC=`which spu-gcc`
+        SPU_CFLAGS="-Wall -O2"
+        EMBEDSPU=`which embedspu`
+        EMBEDSPU_ARGS="-m64"
+      fi
+    fi
     ARCH="PPC"
     if [ $SYS = MACOSX ]
     then
@@ -429,6 +444,15 @@ DEVNULL=$DEVNULL
 ECHON=$ECHON
 EOF
 
+if [ $HAVE_LIBSPE2 = 1 ]; then
+cat >> config.mak << EOF
+HAVE_LIBSPE2=$HAVE_LIBSPE2
+SPU_CC=$SPU_CC
+SPU_CFLAGS=$SPU_CFLAGS
+EMBEDSPU=$EMBEDSPU $EMBEDSPU_ARGS
+EOF
+fi
+
 $ECHON 'CONFIGURE_ARGS=' >> config.mak
 for A in "$@" ; do
     $ECHON " '$A'" >> config.mak
diff --git a/x264.h b/x264.h
index 323f9bb..4170856 100644
--- a/x264.h
+++ b/x264.h
@@ -61,6 +61,7 @@ typedef struct x264_t x264_t;
 #define X264_CPU_STACK_MOD4     0x001000  /* if stack is only mod4 and not mod16 */
 #define X264_CPU_SSE4           0x002000  /* SSE4.1 */
 #define X264_CPU_SSE42          0x004000  /* SSE4.2 */
+#define X264_CPU_CELL_BE        0x008004  /* Cell Broadband Engine (has AltiVec/VMX) */
 
 /* Analyse flags
  */
